/*
Some routines to do parallel random number generation.  The idea is
to take a standard linear congruential algorithm, and have every
processor compute the Pth iterate of that algorithm (where P is the
number of processors.)  If the seeds are set up properly (staggered)
the processors leapfrog over one another, and it is just as good as
having used the basic algorithm on a sequential machine. See Chap 12
of the Book.
*/

#include "comext.h"
#include <math.h>
#include <fcntl.h>
#include <stdio.h>

#define MULT	1103515245
#define ADD	12345
#define MASK	( 0x7fffffff )

static double _twoto31 = 2147483648.0 ;
static long AAA,BBB;

pranset(seed)
/*
	Unlike the sequential algorithm, you MUST call pranset, even if
	you want a seed of 1.  Otherwise all processors will produce the
	same sequence.
*/
long seed;
{
	int proc;

	AAA= 1;
	BBB = 0;
	for (proc=0;proc<nprocs;++proc) {
	   AAA = (MULT * AAA) & MASK;
	   BBB = (MULT * BBB + ADD) & MASK;
	   /* 
	   if ((first) && (proc == procnum)) randx = (AAA*seed + BBB) & MASK;
	   */
	   if (proc == procnum) randx = (AAA*seed + BBB) & MASK;
	}
	/* else just use randx values from previous run */
	/* if (!first) randx = seed; */

	return 0;
}


float ranf()
/*
	Return a random float in [0, 1.0)
*/
{
	float retvalue;

	retvalue = randx / _twoto31;
	randx = (AAA*randx+BBB)&MASK;
	return(retvalue);
}


iranf()
/*
	Return a random int in [0, 2^31)
*/
{
	randx = (AAA*randx+BBB)&MASK;
	return(randx);
}


/**********
* setseed *    J.W.Flower & the infamous Dr. O  March 1985
***********

Used to set the seed for the parallel random number system -
MUST be called or all processors will execute same sequence! - see
Chapter 12 of *THE BOOK*.  Two options:
  i) Input seed from terminal                        
  ii) Read in seed from standard file "Seed".  
ALSO: at the end of production runs, don't forget to store current
seed in "Seed" by running option "d". N.B. You don't need to do this
any more, since the seed is dumped at the end of the backup file 
*/

setseed()
{
    FILE *fp ;
    int mode ;
    long seed ;

    printf("Select mode of seed setting : (0=input,1=file) : ") ;
    fflush(stdout);
    scanf("%d",&mode) ;
    printf("%d\n",mode) ;
    fflush(stdout);

    if (mode == 0) {
      printf("Input seed : ");
      fflush(stdout);
      scanf("%ld",&seed);
    }  
    else {
      if ((fp = fopen("Seed","r")) == NULL) {
        printf("Couldn't find Seedfile...assuming 42 !!\n") ;
        fflush(stdout);
        seed = 42L ;
      }
      else {
        fscanf(fp,"%ld",&seed) ;
        fclose(fp) ; 
      }
    }
    pranset(seed) ;

    printf("Seed set to %ld\n",seed);
    fflush(stdout);

    return(0) ;
}

